# -*- coding: utf-8 -*-

from odoo import api, fields, models, exceptions, _
from datetime import datetime, date, timedelta
from dateutil.relativedelta import relativedelta

from odoo.tools import DEFAULT_SERVER_DATE_FORMAT
from . import consts

import logging
_logger = logging.getLogger(__name__)


class SaleSubscription(models.Model):
    _inherit = 'sale.order'


    def write(self, values):
        result = super(SaleSubscription, self).write(values)

        closed_partnerships = self.env['res.partner']
        closed_integrations = self.env['res.partner']

        if values.get('subscription_state'):
            new_state = values['subscription_state']
            partnership_tag = self.env.ref('openerp_enterprise.odoo_partnership_tag')
            for sub in self:
                integrating_partner_id = values.get('referrer_id', False) or sub.referrer_id.id
                if new_state != '3_progress':
                    if partnership_tag in sub.sale_order_template_id.subscription_tag_ids:
                        closed_partnerships |= sub.partner_id
                    elif sub.referrer_id:
                        closed_integrations |= sub.partner_id
                elif new_state == '3_progress' and integrating_partner_id and integrating_partner_id != sub.partner_id.id:
                    # some partners don't want to be referenced. This tag is used as an opt-out check.
                    if any(t.name == 'PRIVATE_REF' for t in sub.partner_id.website_tag_ids):
                        continue
                    # add website reference for reseller's integrated partner
                    # and propagate integrating partner to assigned partner
                    sub.partner_id.write({
                        'is_published': True,
                        'assigned_partner_id': integrating_partner_id,
                    })

        closed_partnerships._reset_partner_grade()
        closed_integrations._remove_website_reference()

        return result


class ResPartner(models.Model):
    _inherit = 'res.partner'

    # Field not used anywhere in the code but useful for users in their filters
    portal_grade = fields.Char(related='commercial_partner_id.grade_id.name', string="Partner Grade", readonly=True)
    portal_account_mgr_id = fields.Many2one('res.partner', related='commercial_partner_id.user_id.partner_id', string="Account Manager", readonly=True)

    portal_accounting_firm_level = fields.Selection(
        [('3_gold', 'Gold'), ('2_silver', 'Silver'), ('1_ready', 'Ready'), ('0_learning', 'Learning'), ],
        required=False, index=True, tracking=True, string='Accounting Firm Level',
    )

    def _reset_partner_grade(self):
        if not self:
            return

        closed_partnerships = self.env['res.partner']
        partnership_tag = self.env.ref('openerp_enterprise.odoo_partnership_tag')

        sub_data = self.env['sale.order'].sudo().read_group([
            ('is_subscription', '=', True),
            ('partner_id', 'in', self.ids),
            ('subscription_state', 'in', ['3_progress', '4_paused']),
            ('sale_order_template_id.subscription_tag_ids', 'in', [partnership_tag.id]),
        ], ['partner_id'], ['partner_id'])

        sub_count = {row['partner_id'][0]: row['partner_id_count'] for row in sub_data}

        for partner in self:
            # if the partner has no active partnership subscriptions
            if not sub_count.get(partner.id, 0):
                closed_partnerships |= partner

        # unpublish the partner and reset its level and weight
        closed_partnerships.write({
            'is_published': False,
            'grade_id': False,
            'commission_plan_id': False,
            'partner_weight': 0,
        })

        for partner in closed_partnerships:
            partner.message_post(body='Partnership subscription closed: grade level has been reset.')

    def _remove_website_reference(self):
        if not self:
            return

        closed_integrations = self.env['res.partner']

        # double-check ongoing subscriptions
        sub_data = self.env['sale.order'].sudo().read_group([
            ('is_subscription', '=', True),
            ('partner_id', 'in', self.ids),
            ('subscription_state', 'in', ['3_progress', '4_paused']),
            ('referrer_id', '!=', False),
        ], ['partner_id'], ['partner_id'])

        sub_count = {row['partner_id'][0]: row['partner_id_count'] for row in sub_data}

        for partner in self:
            if not sub_count.get(partner.id, 0):
                closed_integrations |= partner

        closed_integrations.write({
            'is_published': False,
            'assigned_partner_id': False,
        })

    def _get_subscriptions_dashboard(self):
        return {
            'current_enterprise_user': self.total_enterprise_users_count,
            'last_12month_enterprise_user': self.new_enterprise_users_count,
        }

    def _get_purchases_dashboard(self):
        last_year = (date.today() - timedelta(days=365)).strftime(DEFAULT_SERVER_DATE_FORMAT)
        purchases = self.env['purchase.order'].sudo().search([
            ('state', 'not in', ['draft', 'cancel']),
            ('partner_id', '=', self.id),
            ('date_order', '>=', last_year),
        ])

        last_12months_purchase_total = 0
        commission_to_receive = 0

        # Amounts should be expressed in the same currency, even when there are multiple POs expressed in different currencies.
        website = self.env['website'].get_current_website()
        currency = website.currency_id

        for p in purchases:
            amount_total = p.currency_id._convert(p.amount_total, currency, self.env.company, fields.Date.today(), round=False)
            last_12months_purchase_total += amount_total
            if p.invoice_status == 'to invoice':
                commission_to_receive += amount_total

        return {
            'orders': purchases,
            'commission_to_receive': commission_to_receive,
            'last_12months_purchase_total': last_12months_purchase_total,
        }

    def _get_grade_dashboard(self):
        certified_experts = []
        partner = self.commercial_partner_id
        partner_grade = partner.grade_id
        partner_to_check = self.commercial_partner_id.child_ids

        if not partner_to_check:  # probably never the case
            partner_to_check = [self.commercial_partner_id]

        for child in partner_to_check:
            certifs = []
            for certif in child.website_tag_ids.filtered(lambda x: x.id in consts.ODOO_VALID_CERT_IDS):
                certifs.append(certif)
            if certifs:
                certified_experts.append({
                    'partner': child,
                    'certifs': certifs
                })

        company_certifs = self.commercial_partner_id.website_tag_ids.filtered(lambda x: x.id in consts.ODOO_VALID_CERT_IDS)
        next_level_com = partner_grade and consts.NEXT_GRADE_COMMISSION_VALUE[partner_grade.id] or 0

        return {
            'ready_grade': consts.READY_GRADE,
            'silver_grade': consts.SILVER_GRADE,
            'gold_grade': consts.GOLD_GRADE,
            'learning_grade': consts.LEARNING_GRADE,
            'partner_grade': partner_grade,
            'company_certif': company_certifs,
            'next_level_com': next_level_com,
            'certified_experts': certified_experts,
            'nbr_certif': len(certified_experts),
            'retention_rate': partner.retention_rate,
        }

    def _get_leads_dashboard(self):
        Lead = self.env['crm.lead'].sudo()
        partner_to_check = self.commercial_partner_id.child_ids.ids
        partner_to_check.append(self.commercial_partner_id.id)

        not_won_opportunities = Lead.search_count([
            ('partner_assigned_id', 'in', partner_to_check),
            ('stage_id', '!=', consts.WON_STAGE),
            ('type', '=', 'opportunity'),
        ])
        leads_by_stage = Lead.read_group([('partner_assigned_id', 'in', partner_to_check), ('type', '=', 'opportunity')], ['stage_id'], ['stage_id'])
        count_by_stage = {row['stage_id'][0]: row['stage_id_count'] for row in leads_by_stage}
        stages = self.env['crm.stage'].browse(count_by_stage.keys())
        return {
            'not_won_opportunities': not_won_opportunities,
            'opportunities': {stage.name: count_by_stage.get(stage.id, 0) for stage in stages},
        }

    @api.constrains('grade_id', 'commission_plan_id')
    def _check_commission_plan_if_grade(self):
        if self.filtered(lambda r: bool(r.grade_id.id) != bool(r.commission_plan_id.id)):
            raise exceptions.ValidationError(_("A partner cannot have a grade without a commission plan, and vice versa."))


class ResPartnerGrade(models.Model):
    _inherit = 'res.partner.grade'

    target_users = fields.Integer(string="Target users")
    target_certif = fields.Integer(string="Target certified experts")
    target_retention_rate = fields.Integer(string="Target Retention Rate")
    next_grade = fields.Many2one("res.partner.grade", string="Next Grade")
